/*
Module : Splasher.cpp
Purpose: A splash screen component for MFC which uses a DIB bitmap instead of a resource. Palette management 
         code is also included so that the bitmap will display using its own optimized palette. Also most
         importantly, the splash screen is shown in its own thread, meaning that it will redraw correctly
         while you app is performing it's startup code and not necessarily pumping the message queue.
Created: PJN / 15-11-1996
History: PJN / 11-12-1997 1) Incoporation of new DIB code provided by the authors own CDibImage class. A static library 
                          version is used by splasher here. If you want to use CDIbImage for anything other than 
                          CSplashThread, then you need to download CDibImage on its own.
                          2) Can now load from resource or bitmap
                          3) General tidy up of the code.
         PJN / 22-3-1998  1) Palette is now correctly initialised on startup
                          2) Code now protects itself if m_SplashScreen cannot be created
         PJN / 22-12-1998 1) Now no longer dependent on CDibImage.
                          2) Unicode Enabled the code,
                          3) General tidy up of the code
                          4) Removed the unnecessary variable m_bCreated
                          5) Fixed a potential race condition in CSplashThread::HideSplash()
         PJN / 01-03-2000 1) Fixed a problem with bitmaps which do not have a palette
                          2) Fixed a problem in Win 98 and Win2000 when the splash screen is closed but the main window 
                          of your app fails to activate. The code now uses AttachThreadInput to synchronise the UI 
                          activities of the main GUI thread and the splash screen thread.
         PJN / 01-01-2001 1) Now includes copyright message in the source code and documentation.
                          2) Modified the way the sample app waits for the thread to exit. It now synchronises with the 
                          exit of the splash screen thread
                          3) Now allows the icon of the splash screen to be specified
                          4) Now allows the splash screen window to be customized at run time through a virtual function 
                          "CSplashThread::CreateSplash". Thanks to Yury Goltsman for suggesting these enhancements.
         PJN / 27-08-2001 1) Splash screen now by default includes the window style WS_EX_TOPMOST
         PJN / 31-08-2001 1) Fixed an assert which was occurring when you brought up the splash window and closed it 
                          really quickly. Thanks to Wanner Robert for spotting this problem.
                          2) Removed a busy loop in HideSplash and used a Win32 event for synchronisation instead.
                          3) Implemented a class factory design pattern to further simplify the use of the splash screen 
                          class. All client code now needs to do is derive a class from CSplashWnd (making sure it's 
                          declared DYNCREATE) and at a minimum just call some of it's methods in the constructor to 
                          determine what it will display at runtime. Then just use CSplashFactory::Create with the name 
                          of your derived class. When you want to close the splash screen just call CSplashFactory::Close. 
                          What could be simpler!!!
         PJN / 03-10-2002 1) Now allows you to specify a transparent color to draw the splash screen with. This allows you 
                          to realize shapes other than the standard rectangle. Thanks to Gero Gerber for this update. Please 
                          note that this support is only available on Windows 2000 or later.
                          2) Made the destructor of CSplashWnd virtual
         PJN / 11-10-2002 1) Fixed up some coding issues reported by BoundsChecker in CSplashWnd::OnPaint.
                          2) Now supports a drop shadow effect on the splash screen (assuming that the client OS is 
                          Windows XP or later).
         PJN / 28-08-2003 1) Removed a number of level 4 warnings.
         PJN / 06-09-2003 1) Fixed a problem where you get a crash in the close method if the window to gain focus was NULL. 
                          Thanks to Darius Thabit for reporting this problem.
         PJN / 09-05-2004 1) Updated the copyright details
                          2) Included a new sample app in the download to demonstrate the correct usage of the splash screen in 
                          a dialog based app.
                          3) Fixed a bug where sometimes the splash screen component would get stuck while waiting for the splash 
                          screen thread to exit. The fix is to request the parent window to exit in CSplashWnd::OnClose. Thanks 
                          to Frederic Metrich for reporting this issue.
         PJN / 09-07-2006 1. Updated the code to clean compile on VC 2005.
                          2. Updated copyright details.  
                          3. Addition of CSPLASHER_EXT_CLASS macro to allow the classes to be easily added to an extension dll.  
                          4. Code now uses newer C++ style casts instead of C style casts.    
                          5. Optimized CSplashWnd constructor code  
                          6. Optimized CSplashThread constructor code  
                          7. Reviewed all TRACE statements for correctness.    
                          8. UI thread which is created by the code is now created at idle priority  
                          9. Updated the documentation to use the same style as the web site.       
         PJN / 22-07-2007 1. Updated copyright details.
                          2. Optimized CSplashFactory constructor code
                          3. CSplashWnd code now uses WM_NCHITTEST logic to support a draggable splash screen
                          4. Made Create and Close methods in CSplashFactory virtual
                          5. CSplashFactory::Create now waits for the worker thread's InitInstance to complete
                          6. Removed unnecessary VERIFY call from CSplashWnd::Create
                          7. CSplashWnd::CreatePaletteFromBitmap now returns a BOOL
                          8. Removed the unnecessary CSplashWnd destructor
                          9. Removed the unnecessary CSplashThread destructor.
                          10. Reviewed all TRACE statements for correctness
                          11. Removed unnecessary use of friend classes
                          12. Updated sample apps to clean compile on VC2005
         PJN / 13-10-2007 1. CSplashFactory::Create now by default creates the splash screen thread at normal priority instead of 
                          idle priority. This helps avoid thread starvation issues when the main thread is busy performing its 
                          startup code. Please be aware of this issue if you modify your applications main thread priority to be 
                          anything other than THREAD_PRIORITY_NORMAL. If you do please make sure that the nPriority value to 
                          CSplashFactory::Create is >= the value you use for your main thread, or otherwise you will starve the 
                          splash screen of CPU cycles on single core processor. Thanks to Ralf Magnussen for reporting this issue.
         PJN / 17-11-2007 1. Fixed a memory leak where the CMySplashWnd instance was not being deleted in the worker thread. 
                          Thanks to Laurent Lhermin for reporting this bug.
         PJN / 17-05-2009 1. Updated copyright details
                          2. Removed VC 6 style class wizard comments from the code.
                          3. Updated code to compile correctly using _ATL_CSTRING_EXPLICIT_CONSTRUCTORS define
                          4. Updated CSplashWnd::SetTransparent to allow the alpha transparency level as well as the flags passed 
                          to SetLayeredWindowAttributes to be completed specified
                          5. Updated code to clean compile on VC 2008.
                          6. Code now compiles cleanly using Code Analysis (/analyze)
                          7. The code now only supports VC 2005 or later.
         PJN / 22-09-2013 1. Updated copyright details
                          2. Updated the code and sample app to clean compile on VC 2012
                          3. Updated CSplashWnd::LoadBitmap to use AfxFindResourceHandle when locating the bitmap. Thanks to Emeric 
                          Maschino for this nice addition.
         PJN / 18-01-2015 1. Updated copyright details. 
                          2. Updated the code and sample app to clean compile on VC 2013
         PJN / 10-01-2015 1. Updated copyright details
                          2. Verified the code compiles cleanly on VC 2015.
                          3. Added SAL annotations to all the code.

Copyright (c) 1996 - 2016 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "Splasher.h"


//////////////////// Macros / Defines /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//Define the various constants we need if not available
#ifndef WS_EX_LAYERED
#define WS_EX_LAYERED 0x00080000
#endif
#ifndef CS_DROPSHADOW
#define CS_DROPSHADOW 0x00020000
#endif


//////////////////// Implementation ///////////////////////////////////////////

BEGIN_MESSAGE_MAP(CSplashWnd, CWnd)
  ON_WM_CREATE()
  ON_WM_PAINT()
  ON_WM_CLOSE()
  ON_WM_NCHITTEST()
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CSplashWnd, CWnd);

CSplashWnd::CSplashWnd() : m_bOKToClose(FALSE),
                           m_nHeight(0),
                           m_nWidth(0),
                           m_bUseFile(FALSE),
                           m_pszResourceName(NULL),
                           m_hIcon(NULL),
                           m_bTransparent(FALSE),
                           m_clrTransparent(RGB(255, 0, 255)),
                           m_bDraggable(TRUE),
                           m_bDropShadow(FALSE)
{
  //Determine if the OS supports layered windows
  HINSTANCE hUser32 = GetModuleHandle(_T("USER32.DLL"));
  if (hUser32 != NULL)
    m_lpfnSetLayeredWindowAttributes = reinterpret_cast<lpfnSetLayeredWindowAttributes>(GetProcAddress(hUser32, "SetLayeredWindowAttributes"));
  else 
    m_lpfnSetLayeredWindowAttributes = NULL;
}

BOOL CSplashWnd::LoadBitmap()
{     
  //Use LoadImage to get the image loaded into a DIBSection
  HBITMAP hBitmap;
  if (m_bUseFile)
    hBitmap = static_cast<HBITMAP>(LoadImage(NULL, m_sFilename, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE | LR_LOADFROMFILE));
  else
    hBitmap = static_cast<HBITMAP>(LoadImage(AfxFindResourceHandle(m_pszResourceName, RT_BITMAP), m_pszResourceName, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_DEFAULTSIZE));

  //Check that we could load it up  
  if (hBitmap == NULL)       
  {
    TRACE(_T("CSplashWnd::LoadBitmap, Failed to load specified bitmap for splash screen\n"));
    return FALSE;
  }

  //Get the width and height of the DIBSection
  BITMAP bm;
  if (!GetObject(hBitmap, sizeof(BITMAP), &bm))
  {
    TRACE(_T("CSplashWnd::LoadBitmap, Failed to get the DIBSection's attributes\n"));
    return FALSE;
  }
  m_nHeight = bm.bmHeight;
  m_nWidth = bm.bmWidth;

  //Covert from the SDK bitmap handle to the MFC equivalent
  return m_Bitmap.Attach(hBitmap);
}

BOOL CSplashWnd::SetTransparent(_In_ COLORREF clrTransparent, _In_ DWORD dwFlags, _In_ BYTE bAlpha)
{
  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  if (m_lpfnSetLayeredWindowAttributes != NULL)
  {
    m_bTransparent = TRUE;
    m_clrTransparent = clrTransparent;
    m_dwSLWAFlags = dwFlags;
    m_bSLWAAlpha = bAlpha;
    bSuccess = TRUE;
  }

  return bSuccess;
}

void CSplashWnd::SetDraggable(_In_ BOOL bDraggable)
{
  m_bDraggable = bDraggable;
}

void CSplashWnd::SetDropShadow(_In_ BOOL bDropShadow)
{
  m_bDropShadow = bDropShadow;
}

BOOL CSplashWnd::CreatePaletteFromBitmap()
{
  //what will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  //Get the color depth of the DIBSection
  BITMAP bm;
  if (m_Bitmap.GetObject(sizeof(BITMAP), &bm))
  {
    //If the DIBSection is 256 color or less, it has a color table
    if ((bm.bmBitsPixel * bm.bmPlanes) <= 8)     
    {
      //Create a memory DC and select the DIBSection into it so that we can get its color table
      CDC memDC;
      memDC.CreateCompatibleDC(NULL);
      CBitmap* pOldBitmap = memDC.SelectObject(&m_Bitmap);

      //Get the DIBSection's color table
      RGBQUAD rgb[256];
      UINT nColors = GetDIBColorTable(memDC.m_hDC, 0, 256, rgb);
      if (nColors)
      {
        //Create a palette from the color table
        ATL::CHeapPtr<BYTE> palette;
        if (palette.Allocate(sizeof(LOGPALETTE) + (nColors*sizeof(PALETTEENTRY))))
        {
          LOGPALETTE* pLogPal = reinterpret_cast<LOGPALETTE*>(palette.m_pData);
          pLogPal->palVersion = 0x300;       
          pLogPal->palNumEntries = 256;

          for (UINT i=0; i<nColors; i++)       
          {
            pLogPal->palPalEntry[i].peRed = rgb[i].rgbRed;
            pLogPal->palPalEntry[i].peGreen = rgb[i].rgbGreen;
            pLogPal->palPalEntry[i].peBlue = rgb[i].rgbBlue;
            pLogPal->palPalEntry[i].peFlags = 0;
          }
          bSuccess = m_Palette.CreatePalette(pLogPal);
          if (!bSuccess)
            TRACE(_T("CSplashWnd::CreatePaletteFromBitmap, Failed to create palette for splash screen\n"));
        }
        else
          TRACE(_T("CSplashWnd::CreatePaletteFromBitmap, Failed to allocate memory for palette\n"));
      }
      else
        TRACE(_T("CSplashWnd::CreatePaletteFromBitmap, Failed to get the DIBSection's color table\n"));
      
      //Clean up
      memDC.SelectObject(pOldBitmap);
    }
  }
  else
    TRACE(_T("CSplashWnd::CreatePaletteFromBitmap, Failed to get the DIBSection's attributes\n"));
  
  if (!bSuccess) //just use a halftone palette as a fallback
  {
    CDC* pRefDC = GetDC();
    bSuccess = m_Palette.CreateHalftonePalette(pRefDC);
    ReleaseDC(pRefDC);     
  }     

  return bSuccess;
}

BOOL CSplashWnd::Create()
{                   
  //Load up the bitmap from file or from resource
  if (!LoadBitmap())
    return FALSE;

  //Modify the owner window of the splash screen to be an invisible WS_POPUP 
  //window so that the splash screen does not appear in the task bar
  LPCTSTR pszWndClassOwner = AfxRegisterWndClass(0, 0, 0, m_hIcon);
  if (!m_wndOwner.CreateEx(0, pszWndClassOwner, _T(""), WS_POPUP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, 0))
  {
    TRACE(_T("CSplashWnd::Create, Failed to create owner window\n"));  
    return FALSE;
  }

  //Create the appropriate window class to use for the splash window
  LPCTSTR pszWndClass = NULL;
  if (m_bDropShadow)
  {
    try
    {
      pszWndClass = AfxRegisterWndClass(CS_DROPSHADOW, AfxGetApp()->LoadStandardCursor(IDC_ARROW), 0, m_hIcon);
    }
    catch(CResourceException* pEx)
    {
      //Avoid the memory leak
      pEx->Delete();

      //Probably because CS_DROPSHADOW is not supported on this OS, try again without it
      pszWndClass = AfxRegisterWndClass(0, AfxGetApp()->LoadStandardCursor(IDC_ARROW), 0, m_hIcon);
    }
  }
  else
    pszWndClass = AfxRegisterWndClass(0, AfxGetApp()->LoadStandardCursor(IDC_ARROW), 0, m_hIcon);

  //Create this window
  if (!CreateEx(WS_EX_TOPMOST, pszWndClass, _T(""), WS_POPUP | WS_VISIBLE, 0, 0, m_nWidth, m_nHeight, m_wndOwner.GetSafeHwnd(), NULL))
  {
    TRACE(_T("CSplashWnd::Create, Failed to create the splash screen\n"));
    return FALSE;
  }

  //Make the window transparent if required to
  if (m_bTransparent && (m_lpfnSetLayeredWindowAttributes != NULL))
  {
    //Add the transparent window style
    LONG nStyle = GetWindowLong(m_hWnd, GWL_EXSTYLE);
    SetWindowLong(m_hWnd, GWL_EXSTYLE, nStyle | WS_EX_LAYERED);

    //Make the Window transparent using the specifed color
    m_lpfnSetLayeredWindowAttributes(m_hWnd, m_clrTransparent, m_bSLWAAlpha, m_dwSLWAFlags);
  }

  //Create the palette, We need to do this after the window is created because
  //we may need to access the DC associated with it
  if (!CreatePaletteFromBitmap())
    return FALSE;

  //realize the bitmap's palette into the DC
  OnQueryNewPalette();

  return TRUE;
}

int CSplashWnd::OnCreate(_In_ LPCREATESTRUCT lpCreateStruct)
{
  //Let the parent class do its thing
  if (CWnd::OnCreate(lpCreateStruct) == -1)
    return -1;

  //Center the splash window on the screen
  CenterWindow();

  return 0;
}

void CSplashWnd::OnPaint()
{
  CPaintDC dc(this);
  
  //select the palette and bitmap into the memory DC and then bitblit it to the screen DC
  CDC memDC;
  memDC.CreateCompatibleDC(&dc);
  CBitmap* pOldBitmap = memDC.SelectObject(&m_Bitmap);
  CPalette* pOldPalette = NULL;
  if (m_Palette.m_hObject != NULL)
    pOldPalette = dc.SelectPalette(&m_Palette, FALSE);
  dc.RealizePalette();
  dc.BitBlt(0, 0, m_nWidth, m_nHeight, &memDC, 0, 0, SRCCOPY);

  //Restore the DC
  memDC.SelectObject(pOldBitmap);         
  if (pOldPalette != NULL)
    dc.SelectPalette(pOldPalette, FALSE);
}

void CSplashWnd::OnClose() 
{
  if (m_bOKToClose) 
  {
    //Just request the parent window to close which will cause this window to be destroyed
    m_wndOwner.PostMessage(WM_CLOSE);
  }
}

BOOL CSplashWnd::SelRelPal(_In_ BOOL bForceBkgnd)
{
  //We are going active, so realize our palette.
  CDC* pDC = GetDC();

  CPalette* pOldPal = pDC->SelectPalette(&m_Palette, bForceBkgnd);
  UINT nRemappedColors = pDC->RealizePalette();
  pDC->SelectPalette(pOldPal, TRUE);
  pDC->RealizePalette();

  ReleaseDC(pDC);

  //If any colors have changed or we are in the
  //background, repaint the lot.
  if (nRemappedColors || bForceBkgnd) 
    InvalidateRect(NULL, TRUE);
  
  return (nRemappedColors != 0); //TRUE if some colors changed.
}

void CSplashWnd::OnPaletteChanged(_In_ CWnd* pFocusWnd)
{
  //See if the change was caused by us and ignore it if not
  if (pFocusWnd != this) 
    SelRelPal(TRUE); //Realize in the background. 
}

BOOL CSplashWnd::OnQueryNewPalette()
{
  return SelRelPal(FALSE); //Realize in the foreground.
}

void CSplashWnd::SetBitmapToUse(_In_ const CString& sFilename)
{
  m_bUseFile = TRUE;
  m_sFilename = sFilename;
}

void CSplashWnd::SetBitmapToUse(_In_ UINT nResourceID)
{
  m_bUseFile = FALSE;
  m_pszResourceName = MAKEINTRESOURCE(nResourceID);
}

void CSplashWnd::SetBitmapToUse(_In_ LPCTSTR pszResourceName)
{
  m_bUseFile = FALSE;
  m_pszResourceName = pszResourceName;
}

void CSplashWnd::SetIcon(_In_ HICON hIcon)
{
  //Destroy the icon if we have one already loaded
  if (m_hIcon != NULL)
    DestroyIcon(m_hIcon);
     
  m_hIcon = hIcon; 
}

LRESULT CSplashWnd::OnNcHitTest(_In_ CPoint point)
{
  if (m_bDraggable)
    return HTCAPTION; 
  else
    return CWnd::OnNcHitTest(point);
}


BEGIN_MESSAGE_MAP(CSplashThread, CWinThread)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CSplashThread, CWinThread)

CSplashThread::CSplashThread() : m_pRuntimeClassSplashWnd(NULL),
                                 m_pSplashScreen(NULL),
                                 m_bInitOK(FALSE)
{
}

BOOL CSplashThread::InitInstance()
{
  //Validate our parameters
  AFXASSUME(m_pRuntimeClassSplashWnd != NULL);

  //Create the splash screen window object
  m_pSplashScreen = static_cast<CSplashWnd*>(m_pRuntimeClassSplashWnd->CreateObject());
  m_pMainWnd = m_pSplashScreen;
  if (m_pSplashScreen == NULL)
  {
    TRACE(_T("CSplashThread::InitInstance, Failed to instantiate CSplashWnd object\n"));
    m_evtInitInstanceRun.SetEvent();
    return FALSE;
  }
  
  //Validate that it is derived from CSplashWnd
  ASSERT(m_pSplashScreen->IsKindOf(RUNTIME_CLASS(CSplashWnd)));

  //Create the HWND of it
  if (!m_pSplashScreen->Create())
  {
    TRACE(_T("CSplashThread::InitInstance, Failed to create CSplashWnd object\n"));

    //Tidy up the heap memory we have allocated
    delete m_pSplashScreen;
    m_pSplashScreen = NULL;

    m_evtInitInstanceRun.SetEvent();
    return FALSE;
  }

  //Signal the event which indicates that this InitInstance function has run
  m_bInitOK = TRUE;
  m_evtInitInstanceRun.SetEvent();

  return TRUE;
}

int CSplashThread::ExitInstance()
{
  //Free up our splash screen instance if created
  if (m_pSplashScreen != NULL)
  {
    delete m_pSplashScreen;
    m_pSplashScreen = NULL;
  }

  //Let the base class do its thing
  return CWinThread::ExitInstance();
}


CSplashFactory::CSplashFactory() : m_pSplashThread(NULL)
{
}

CSplashFactory::~CSplashFactory()
{
  //Just incase client code forgot to call Close, we do it anyway
  Close();
}

BOOL CSplashFactory::Create(_In_ CRuntimeClass* pRuntimeClassSplashWnd, _In_ int nPriority)
{
  //Validate our parameters
  ASSERT(m_pSplashThread == NULL);
  ASSERT(pRuntimeClassSplashWnd != NULL);

  //Create the GUI worker thread
  m_pSplashThread = static_cast<CSplashThread*>(AfxBeginThread(RUNTIME_CLASS(CSplashThread), nPriority, 0, CREATE_SUSPENDED));
  if (m_pSplashThread == NULL)
  {
    TRACE(_T("CSplashFactory::Create, Failed to create splash screen thread\n"));
    return FALSE;
  }

  ASSERT(m_pSplashThread->IsKindOf(RUNTIME_CLASS(CSplashThread)));
  m_pSplashThread->m_pRuntimeClassSplashWnd = pRuntimeClassSplashWnd;
  m_pSplashThread->m_bAutoDelete = FALSE; //We are in charge of the deletion of the thread
  m_pSplashThread->ResumeThread();        //Resume the thread now that we have set it up 

  //Wait until the thread's InitInstance code has completed. Note this is important because code in CSplashFactory::Close
  //can execute after Create returns and we want to ensure that "m_pSplashThread->m_pSplashScreen" is not modified from
  //the worker thread while we access it from the main UI thread
  WaitForSingleObject(m_pSplashThread->m_evtInitInstanceRun, INFINITE);
  
  //Get the return value from InitInstance
  return m_pSplashThread->m_bInitOK;
}

BOOL CSplashFactory::Close(_In_opt_ CWnd* pWndToGainFocus)
{
  //What will be the return value from this function (assume the worst)
  BOOL bSuccess = FALSE;

  if (m_pSplashThread != NULL)
  {
    //Attach this threads UI state to the worker threads one, This will ensure that 
    //the activation state is managed consistenly across the two threads
    ASSERT(AfxGetApp() != NULL);
    AttachThreadInput(AfxGetApp()->m_nThreadID, m_pSplashThread->m_nThreadID, TRUE);

    //Set the focus back to the correct window
    CWnd* pWndFocus = pWndToGainFocus;
    if (pWndFocus == NULL)
      pWndFocus = AfxGetMainWnd();
    if (pWndFocus != NULL)
    {
      pWndFocus->SetFocus();
      pWndFocus->SetForegroundWindow();
    }

    //If the worker thread has a splash screen then ask it to close before trying to close it. See comment in 
    //CSplashFactory::Create about why can can safely dot into a pointer set in another thread
    if (m_pSplashThread->m_pSplashScreen)
    {
      m_pSplashThread->m_pSplashScreen->SetOKToClose();
      m_pSplashThread->m_pSplashScreen->PostMessage(WM_CLOSE);
    }

    //Wait for the worker thread to exit and tidy up its memory
    WaitForSingleObject(m_pSplashThread->m_hThread, INFINITE);
    delete m_pSplashThread;
    m_pSplashThread = NULL;

    bSuccess = TRUE;
  }

  return bSuccess;
}
